APPLICATION_PRETTY_NAME = "Mindfulness at the Computer"
APPLICATION_NAME = "mindfulness-at-the-computer"
APPLICATION_VERSION = "1.0.0-alpha.9"
SHORT_DESCR_STR = "Helps you stay mindful of your breathing while using your computer."
